package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.CastExpression;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.OperatorExpression;
import org.eclipse.jdt.internal.compiler.codegen.BranchLabel;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.impl.BooleanConstant;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class UnaryExpression extends OperatorExpression {
  public Expression expression;
  
  public Constant optimizedBooleanConstant;
  
  public UnaryExpression(Expression expression, int operator) {
    this.expression = expression;
    this.bits |= operator << 6;
  }
  
  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
    this.expression.checkNPE(currentScope, flowContext, flowInfo);
    if ((this.bits & 0xFC0) >> 6 == 11) {
      flowContext.tagBits ^= 0x4;
      flowInfo = this.expression
        .analyseCode(currentScope, flowContext, flowInfo)
        .asNegatedCondition();
      flowContext.tagBits ^= 0x4;
      return flowInfo;
    } 
    return this.expression
      .analyseCode(currentScope, flowContext, flowInfo);
  }
  
  public Constant optimizedBooleanConstant() {
    return (this.optimizedBooleanConstant == null) ? 
      this.constant : 
      this.optimizedBooleanConstant;
  }
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
    BranchLabel falseLabel;
    int pc = codeStream.position;
    if (this.constant != Constant.NotAConstant) {
      if (valueRequired)
        codeStream.generateConstant(this.constant, this.implicitConversion); 
      codeStream.recordPositionsFrom(pc, this.sourceStart);
      return;
    } 
    switch ((this.bits & 0xFC0) >> 6) {
      case 11:
        switch ((this.expression.implicitConversion & 0xFF) >> 4) {
          case 5:
            this.expression.generateOptimizedBoolean(
                currentScope, 
                codeStream, 
                null, 
                falseLabel = new BranchLabel(codeStream), 
                valueRequired);
            if (valueRequired) {
              codeStream.iconst_0();
              if (falseLabel.forwardReferenceCount() > 0) {
                BranchLabel endifLabel;
                codeStream.goto_(endifLabel = new BranchLabel(codeStream));
                codeStream.decrStackSize(1);
                falseLabel.place();
                codeStream.iconst_1();
                endifLabel.place();
              } 
              break;
            } 
            falseLabel.place();
            break;
        } 
        break;
      case 12:
        switch ((this.expression.implicitConversion & 0xFF) >> 4) {
          case 10:
            this.expression.generateCode(currentScope, codeStream, valueRequired);
            if (valueRequired) {
              codeStream.iconst_m1();
              codeStream.ixor();
            } 
            break;
          case 7:
            this.expression.generateCode(currentScope, codeStream, valueRequired);
            if (valueRequired) {
              codeStream.ldc2_w(-1L);
              codeStream.lxor();
            } 
            break;
        } 
        break;
      case 13:
        if (this.constant != Constant.NotAConstant) {
          if (valueRequired)
            switch ((this.expression.implicitConversion & 0xFF) >> 4) {
              case 10:
                codeStream.generateInlinedValue(this.constant.intValue() * -1);
                break;
              case 9:
                codeStream.generateInlinedValue(this.constant.floatValue() * -1.0F);
                break;
              case 7:
                codeStream.generateInlinedValue(this.constant.longValue() * -1L);
                break;
              case 8:
                codeStream.generateInlinedValue(this.constant.doubleValue() * -1.0D);
                break;
            }  
          break;
        } 
        this.expression.generateCode(currentScope, codeStream, valueRequired);
        if (valueRequired)
          switch ((this.expression.implicitConversion & 0xFF) >> 4) {
            case 10:
              codeStream.ineg();
              break;
            case 9:
              codeStream.fneg();
              break;
            case 7:
              codeStream.lneg();
              break;
            case 8:
              codeStream.dneg();
              break;
          }  
        break;
      case 14:
        this.expression.generateCode(currentScope, codeStream, valueRequired);
        break;
    } 
    if (valueRequired)
      codeStream.generateImplicitConversion(this.implicitConversion); 
    codeStream.recordPositionsFrom(pc, this.sourceStart);
  }
  
  public void generateOptimizedBoolean(BlockScope currentScope, CodeStream codeStream, BranchLabel trueLabel, BranchLabel falseLabel, boolean valueRequired) {
    if (this.constant != Constant.NotAConstant && this.constant.typeID() == 5) {
      super.generateOptimizedBoolean(
          currentScope, 
          codeStream, 
          trueLabel, 
          falseLabel, 
          valueRequired);
      return;
    } 
    if ((this.bits & 0xFC0) >> 6 == 11) {
      this.expression.generateOptimizedBoolean(
          currentScope, 
          codeStream, 
          falseLabel, 
          trueLabel, 
          valueRequired);
    } else {
      super.generateOptimizedBoolean(
          currentScope, 
          codeStream, 
          trueLabel, 
          falseLabel, 
          valueRequired);
    } 
  }
  
  public StringBuffer printExpressionNoParenthesis(int indent, StringBuffer output) {
    output.append(operatorToString()).append(' ');
    return this.expression.printExpression(0, output);
  }
  
  public TypeBinding resolveType(BlockScope scope) {
    int tableId;
    boolean expressionIsCast;
    if (expressionIsCast = this.expression instanceof CastExpression)
      this.expression.bits |= 0x20; 
    TypeBinding expressionType = this.expression.resolveType(scope);
    if (expressionType == null) {
      this.constant = Constant.NotAConstant;
      return null;
    } 
    int expressionTypeID = expressionType.id;
    boolean use15specifics = ((scope.compilerOptions()).sourceLevel >= 3211264L);
    if (use15specifics && 
      !expressionType.isBaseType())
      expressionTypeID = (scope.environment().computeBoxingType(expressionType)).id; 
    if (expressionTypeID > 15) {
      this.constant = Constant.NotAConstant;
      scope.problemReporter().invalidOperator(this, expressionType);
      return null;
    } 
    switch ((this.bits & 0xFC0) >> 6) {
      case 11:
        tableId = 0;
        break;
      case 12:
        tableId = 10;
        break;
      default:
        tableId = 13;
        break;
    } 
    int operatorSignature = OperatorSignatures[tableId][(expressionTypeID << 4) + expressionTypeID];
    this.expression.computeConversion((Scope)scope, TypeBinding.wellKnownType((Scope)scope, operatorSignature >>> 16 & 0xF), expressionType);
    this.bits |= operatorSignature & 0xF;
    switch (operatorSignature & 0xF) {
      case 5:
        this.resolvedType = (TypeBinding)TypeBinding.BOOLEAN;
        break;
      case 3:
        this.resolvedType = (TypeBinding)TypeBinding.BYTE;
        break;
      case 2:
        this.resolvedType = (TypeBinding)TypeBinding.CHAR;
        break;
      case 8:
        this.resolvedType = (TypeBinding)TypeBinding.DOUBLE;
        break;
      case 9:
        this.resolvedType = (TypeBinding)TypeBinding.FLOAT;
        break;
      case 10:
        this.resolvedType = (TypeBinding)TypeBinding.INT;
        break;
      case 7:
        this.resolvedType = (TypeBinding)TypeBinding.LONG;
        break;
      default:
        this.constant = Constant.NotAConstant;
        if (expressionTypeID != 0)
          scope.problemReporter().invalidOperator(this, expressionType); 
        return null;
    } 
    if (this.expression.constant != Constant.NotAConstant) {
      this.constant = 
        Constant.computeConstantOperation(
          this.expression.constant, 
          expressionTypeID, (
          this.bits & 0xFC0) >> 6);
    } else {
      this.constant = Constant.NotAConstant;
      if ((this.bits & 0xFC0) >> 6 == 11) {
        Constant cst = this.expression.optimizedBooleanConstant();
        if (cst != Constant.NotAConstant)
          this.optimizedBooleanConstant = BooleanConstant.fromValue(!cst.booleanValue()); 
      } 
    } 
    if (expressionIsCast)
      CastExpression.checkNeedForArgumentCast(scope, tableId, operatorSignature, this.expression, expressionTypeID); 
    return this.resolvedType;
  }
  
  public void traverse(ASTVisitor visitor, BlockScope blockScope) {
    if (visitor.visit(this, blockScope))
      this.expression.traverse(visitor, blockScope); 
    visitor.endVisit(this, blockScope);
  }
}
